library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;

entity PushButtons is
	generic (
		CLOCK_RATE		: integer	:= 50000000;
		ACTIVE_HIGH		: integer	:= 0;
		NUM_SWITCHES	: integer	:= 4
	);
	port (
		clk			: in	std_logic;
		reset			: in	std_logic;
		
		buttons		: in	std_logic_vector(NUM_SWITCHES-1 downto 0);
		
		-- Avalon Slave
		address		: in	std_logic;
		writedata	: in	std_logic_vector(31 downto 0);
		readdata		: out	std_logic_vector(31 downto 0);
		write			: in	std_logic;
		read			: in	std_logic;
		
		irq			: out	std_logic
	);
end PushButtons;

architecture RTL of PushButtons is

	constant SAMPLE_TIME_MAX	: integer := CLOCK_RATE / 1000;
	constant ZEROS					: std_logic_vector(NUM_SWITCHES-1 downto 0) := (others => '0');
	
	signal buttonIn				: std_logic_vector(NUM_SWITCHES-1 downto 0);	
	signal buttonSample			: std_logic;
	signal buttonState			: std_logic_vector(NUM_SWITCHES-1 downto 0);	
	signal buttonIRQ				: std_logic_vector(NUM_SWITCHES-1 downto 0);
	signal buttonIRQEn			: std_logic_vector(NUM_SWITCHES-1 downto 0);	
	
begin

irq <= '0' when (buttonIRQ = ZEROS) else '1';

-- Invert buttons if active low
buttonIn <= not buttons when (ACTIVE_HIGH = 0) else buttons;

AvalonInterface : process(clk)
begin
	if (rising_edge(clk)) then
		if (reset = '1') then
			readdata <= (others => '0');
		else
			if (read = '1') then
				case address is
					when '0' =>
						readData(NUM_SWITCHES-1 downto 0) <= buttonIRQ;
					when '1' =>
						readData(NUM_SWITCHES-1 downto 0) <= buttonState;
					when others =>
						null;
				end case;
			end if;
		end if;
	end if;
end process;

-- This generates a sample pulse every 1 ms
SampleGen : process(clk)
	variable count : integer range 0 to SAMPLE_TIME_MAX;
begin
	if (rising_edge(clk)) then
		if (reset = '1') then
			count := 0;
			buttonSample <= '0';
		else
			if (count = SAMPLE_TIME_MAX) then
				count := 0;
				buttonSample <= '1';
			else
				count := count + 1;
				buttonSample <= '0';			
			end if;
		end if;
	end if;
end process;

-- Basic button debouncing. The button is sampled every 1 ms into a shift regsiter
-- Consecutive high samples sets the button state high
DebounceGen : for sw in 0 to NUM_SWITCHES-1 generate
	signal shift : std_logic_vector(0 to 5);
begin
	Debounce : process(clk)
	begin
		if (rising_edge(clk)) then
			if (reset = '1') then
				shift <= (others => '0');
				buttonState(sw) <= '0';
				buttonIRQ(sw) <= '0';
			else
				if (buttonSample = '1') then
					shift <= buttonIn(sw) & shift(0 to 4);
				end if;
				
				if (write = '1') then
					case address is
						when '0' =>
							buttonIRQ(sw) <= buttonIRQ(sw) and (not writedata(sw));
						when '1' =>
							buttonIRQEn(sw) <= writedata(sw);
						when others =>
							null;
					end case;
				end if;
				
				if (shift(3 to 5) = b"111") then
					if (buttonState(sw) = '0' and buttonIRQEn(sw)='1') then
						buttonIRQ(sw) <= '1';
					end if;
					buttonState(sw) <= '1';
				else
					buttonState(sw) <= '0';	
				end if;
			end if;
		end if;			
	end process;
end generate;

end RTL;
